@@ -26,6 +26,7 @@ use Class::Std;
 use Config::Any;
 use Data::Dumper;
 use Date::Manip;
+use Digest::MD5;
 use English qw( -no_match_vars);
 use File::Basename;
 use File::Glob;
@@ -40,7 +41,7 @@ use Unicode::String;
 
 {    # BEGIN CLASS
 
-    use version; our $VERSION = qv('3.4.0.5');
+    use version; our $VERSION = qv('3.4.0.9');
     use re 'taint';    # Keep data captured by parens tainted
     delete @ENV{qw(PATH IFS CDPATH ENV BASH_ENV)};    # Make %ENV safe
 
@@ -191,7 +192,6 @@ use Unicode::String;
         # +------------------------------------------------------------------+
         # | prepare
         my $logger = get_logger('CipUX');
-        $logger->debug('BEGIN');
 
         my $config;
 
@@ -226,8 +226,6 @@ use Unicode::String;
             $self->exc( { msg => $msg, value => 'config_hr1' } );
         }
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return ( $cfg0_hr, $cfg1_hr, $cfg2_hr );
@@ -249,7 +247,6 @@ use Unicode::String;
         # | main
         my $logger = get_logger('CipUX');
 
-        $logger->debug('BEGIN');
         $logger->debug( 'use configuration file: ', $cfg );
 
         #$cfg = './' . $cfg if not $cfg =~ m%^/%;
@@ -278,8 +275,6 @@ use Unicode::String;
             } ## end if ( not my $r = do $cfg)
         } ## end if ( -e $cfg )
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $config_hr0, $config_hr1, $config_hr2;
@@ -298,7 +293,6 @@ use Unicode::String;
         return if not defined $option;
 
         my $logger = get_logger('CipUX');
-        $logger->debug('BEGIN');
         $logger->debug( '> attr_hr: ',
             { filter => \&Dumper, value => $attr_hr } );
         $logger->debug( '> opt_hr : ',
@@ -323,12 +317,12 @@ use Unicode::String;
             $opt_hr->{x}          = 1;
             $opt_hr->{mattrvalue} = 1;
 
-            $logger->debug('END');
+            $logger->debug('end 1');
 
             return $attr_hr;
         } ## end if ( ( $opt eq 'x' or ...
 
-        $logger->debug('END');
+        $logger->debug('end 2');
 
         return;
 
@@ -346,7 +340,6 @@ use Unicode::String;
         return if not defined $option;
 
         my $logger = get_logger('CipUX');
-        $logger->debug('BEGIN');
         $logger->debug( '> attr_hr: ',
             { filter => \&Dumper, value => $attr_hr } );
         $logger->debug( '> opt_hr : ',
@@ -372,12 +365,12 @@ use Unicode::String;
             $opt_hr->{y}         = 1;
             $opt_hr->{attrvalue} = 1;
 
-            $logger->debug('END');
+            $logger->debug('end 1');
 
             return $attr_hr;
         } ## end if ( ( $opt eq 'y' or ...
 
-        $logger->debug('END');
+        $logger->debug('end 2');
 
         return;
 
@@ -679,7 +672,6 @@ use Unicode::String;
         # | main
         my $logger = get_logger('CipUX');
 
-        $logger->debug('BEGIN');
         $logger->debug( '> script: ', $script );
         if ( defined $opt_hr ) {
             $logger->debug( '> opt_hr: ',
@@ -756,8 +748,6 @@ use Unicode::String;
             } ## end foreach my $n ( @{ $logic_hr...
         } ## end foreach my $s ( sort keys %...
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return;
@@ -883,21 +873,26 @@ use Unicode::String;
             || $self->perr('password');
         my $mode = $self->l( $arg_r->{mode} ) || $self->perr('mode');
 
+        my $l = get_logger(__PACKAGE__);
         if ( $mode eq 'crypt' ) {
+            $l->debug("crypt|md5 mode eq [$mode]");
             my $modsalt = join $EMPTY_STRING,
                 @MODSALT_CHARS[ rand $MODSALT_BASE, rand $MODSALT_BASE ];
+            $l->debug("modsalt  [$modsalt]");
 
             $password = crypt $password, $modsalt;
+            $l->debug("password [$password]");
 
         }
         elsif ( $mode eq 'md5' ) {
 
-            # use Digest::MD5 qw(md5);
-            #my $ctx = Digest::MD5->new;
-            #$ctx->add($password);
-            #$password = encode_base64( '{MD5}' . $ctx->b64digest );
+            use Digest::MD5 qw(md5);
+            my $ctx = Digest::MD5->new;
+            $ctx->add($password);
+            $password = encode_base64( '{MD5}' . $ctx->b64digest );
+            $l->debug("password [$password]");
 
-            $self->exc( { msg => 'not supported', value => $mode } );
+            #$self->exc( { msg => 'not supported', value => $mode } );
 
         }
         else {
@@ -1050,7 +1045,8 @@ use Unicode::String;
 
         # OK: .cnf .json .jsn .xml .yml .yaml .ini .pl .perl ...
         # BAD: .conf
-        my @extension = grep { !m/conf/smx } @{ Config::Any->extensions };
+        #my @extension = grep { !m/conf/smx } @{ Config::Any->extensions };
+        my @extension = qw(ini perl);    # reduced to used .ini and .perl
 
         # +------------------------------------------------------------------+
         # | API
@@ -1061,7 +1057,7 @@ use Unicode::String;
     # || cfg                                                                ||
     # +======================================================================+
     # NEW config space, will be used after 3.4.0.0
-    sub cfg {    ## no critic (Subroutines::ProhibitExcessComplexity)
+    sub cfg {
 
         # +------------------------------------------------------------------+
         # | API
@@ -1069,18 +1065,17 @@ use Unicode::String;
 
         my $pkg = exists $arg_r->{pkg} ? $arg_r->{pkg} : 'cipux';
         my $sub = exists $arg_r->{sub} ? $arg_r->{sub} : $EMPTY_STRING;
-        my $cfg = exists $arg_r->{cfg} ? $arg_r->{cfg} : undef;
+        my $cfg = exists $arg_r->{cfg} ? $arg_r->{cfg} : undef; # only one cfg
         my $cache_dir
-            = (exists $arg_r->{cache_dir}
-            and defined $arg_r->{cache_dir}
-            and $arg_r->{cache_dir} ne $EMPTY_STRING)
+            = (     exists $arg_r->{cache_dir}
+                and defined $arg_r->{cache_dir}
+                and $arg_r->{cache_dir} ne $EMPTY_STRING )
             ? $self->l( $arg_r->{cache_dir} )
             : $CACHE_DIR;
 
         # +------------------------------------------------------------------+
         # | prepare
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
         $logger->debug("pkg [$pkg]");
         $logger->debug("sub [$sub]");
 
@@ -1095,59 +1090,40 @@ use Unicode::String;
 
         $self->create_cache_dir_if_not_present( { cache_dir => $cache_dir } );
 
-        if ( -e "$cache_dir/$cfgbase" and not defined $cfg ) {
-            $logger->debug("use disk: $cache_dir/$cfgbase");
-            my $cfg_hr = retrieve("$cache_dir/$cfgbase")
-                or croak "Can not load $cfgbase in $cache_dir/$cfgbase";
+        # determine cfg space quantity
+        my $loadcfg_ar = $self->iterate_config_space(
+            {
+                cfg     => $cfg,
+                cfgbase => $cfgbase,
+            }
+        );
+        $logger->debug( 'loadcfg_ar: ',
+            { filter => \&Dumper, value => $loadcfg_ar } );
+
+        # determine cfg space quality
+        my $clean = $self->evaluate_config_space(
+            {
+                loadcfg_ar => $loadcfg_ar,
+                cache_dir  => $cache_dir,
+                cfgbase    => $cfgbase,
+            }
+        );
+
+        if ( -e "$cache_dir/$cfgbase.cache" and $clean and not defined $cfg )
+        {
+            $logger->debug("use disk: $cache_dir/$cfgbase.cache");
+            my $cfg_hr = retrieve("$cache_dir/$cfgbase.cache")
+                or croak
+                "Can not load $cfgbase.cache in $cache_dir/$cfgbase.cache";
             return $cfg_hr;
         }
 
         $logger->debug("use Config::Any $cfgbase");
 
-        my @extension = $self->cfg_ext();
-
-        my @suffix    = ();
-        my @cfg_space = ();
-
-        if ( defined $cfg ) {
-            @cfg_space = ($cfg);
-            $logger->debug("add to config space [$cfg]");
-        }
-        else {
-            @suffix = (
-                "/usr/share/cipux/etc/$cfgbase.",
-                "/usr/share/cipux/etc/$cfgbase.d/*.",
-                "/etc/cipux/$cfgbase.",
-                "/etc/cipux/$cfgbase.d/*.",
-                "~/.cipux/$cfgbase.",
-            );
-
-            foreach my $s (@suffix) {
-                foreach my $e (@extension) {
-                    $logger->debug("add to config space [$s$e]");
-                    push @cfg_space, $s . $e;
-                }
-            }
-        }
-        my @filename = ();
-        foreach my $g (@cfg_space) {
-            $logger->debug("glob [$g]");
-            my @f = sort glob $g;
-            push @filename, @f;
-        }
-
-        my @loadcfg = ();
-        foreach my $f (@filename) {
-            $f = $self->l($f);
-            if ( -e $f ) {
-                $logger->debug("add file [$f] to cfg space");
-                push @loadcfg, $f;
-            }
-        }
-
         # +------------------------------------------------------------------+
         # | main
-        my $cfg_hr = Config::Any->load_files(
+        my @loadcfg = @{$loadcfg_ar};
+        my $cfg_hr  = Config::Any->load_files(
             {
                 files           => \@loadcfg,
                 use_ext         => 1,
@@ -1168,11 +1144,9 @@ use Unicode::String;
             $merged_hr = merge( $merged_hr, $cfg_hr->{$filename} );
         }
 
-        store( $merged_hr, "$cache_dir/$cfgbase" )
+        store( $merged_hr, "$cache_dir/$cfgbase.cache" )
             or croak "Can not save $cfgbase in $cache_dir/$cfgbase";
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $merged_hr;
@@ -1184,9 +1158,9 @@ use Unicode::String;
         my ( $self, $arg_r ) = @_;
 
         my $cache_dir
-            = (exists $arg_r->{cache_dir}
-            and defined $arg_r->{cache_dir}
-            and $arg_r->{cache_dir} ne $EMPTY_STRING )
+            = (     exists $arg_r->{cache_dir}
+                and defined $arg_r->{cache_dir}
+                and $arg_r->{cache_dir} ne $EMPTY_STRING )
             ? $self->l( $arg_r->{cache_dir} )
             : $CACHE_DIR;
 
@@ -1220,6 +1194,122 @@ use Unicode::String;
 
     }
 
+    sub iterate_config_space {
+
+        my ( $self, $arg_r ) = @_;
+        my $cfg
+            = ( exists $arg_r->{cfg} )
+            ? $self->l( $arg_r->{cfg} )
+            : $self->perr('cfg');
+        my $cfgbase
+            = ( exists $arg_r->{cfgbase} )
+            ? $self->l( $arg_r->{cfgbase} )
+            : $self->perr('cfgbase');
+
+        my $l         = get_logger(__PACKAGE__);
+        my @extension = $self->cfg_ext();
+        my @suffix    = ();
+        my @cfg_space = ();
+
+        if ( defined $cfg ) {
+            @cfg_space = ($cfg);
+            $l->debug("add to config space [$cfg]");
+        }
+        else {
+            @suffix = (
+                "/usr/share/cipux/etc/$cfgbase.",
+                "/usr/share/cipux/etc/$cfgbase.d/*.",
+                "/etc/cipux/$cfgbase.",
+                "/etc/cipux/$cfgbase.d/*.",
+                "~/.cipux/$cfgbase.",
+            );
+
+            foreach my $s (@suffix) {
+                foreach my $e (@extension) {
+                    $l->debug("add to config space [$s$e]");
+                    push @cfg_space, $s . $e;
+                }
+            }
+        }
+        my @filename = ();
+        foreach my $g (@cfg_space) {
+            $l->debug("glob [$g]");
+            my @f = sort glob $g;
+            push @filename, @f;
+        }
+
+        my @loadcfg = ();
+        foreach my $f (@filename) {
+            $f = $self->l($f);
+            if ( -e $f ) {
+                $l->debug("add file [$f] to cfg space");
+                push @loadcfg, $f;
+            }
+        }
+        return \@loadcfg;
+    }
+
+    sub evaluate_config_space {
+
+        my ( $self, $arg_r ) = @_;
+        my $loadcfg_ar
+            = ( exists $arg_r->{loadcfg_ar} )
+            ? $self->a( $arg_r->{loadcfg_ar} )
+            : $self->perr('loadcfg_ar');
+        my $cache_dir
+            = ( exists $arg_r->{cache_dir} )
+            ? $self->l( $arg_r->{cache_dir} )
+            : $self->perr('cache_dir');
+        my $cfgbase
+            = ( exists $arg_r->{cfgbase} )
+            ? $self->l( $arg_r->{cfgbase} )
+            : $self->perr('cfgbase');
+        my $l = get_logger(__PACKAGE__);
+
+        my $eval_dir = "$cache_dir/$cfgbase";
+        $self->create_cache_dir_if_not_present( { cache_dir => $eval_dir } );
+
+        my $dirty   = 0;
+        my %ndigest = ();
+        my %odigest = ();
+        foreach my $f ( @{$loadcfg_ar} ) {
+
+            #  /etc/cipux/cipux-cat-web.ini
+            $l->debug("evaluate cfg file [$f]");
+
+            # calc md5 of cfg file
+            open my $RF1, q{<}, $f or croak "Can not open $f for reading!";
+            binmode $RF1;
+            $ndigest{$f} = Digest::MD5->new->addfile($RF1)->hexdigest;
+            close $RF1 or croak "Unable to close $!";
+
+            #|  cache_dir  |cfgbase|name
+            #/var/cache/cipux/cipux/3c9f65e4f1c5d05638f63da289e78eb3
+            my $fn = "$eval_dir/$ndigest{$f}"; 
+            $l->debug("evaluate md5 file [$fn]");
+
+            if ( -e $fn ) {
+
+                # clean
+                $l->debug("[$f] found to be clean");
+            }
+            else {
+                $l->debug("[$f] found to be dirty");
+                $dirty = 1;
+
+                # overwrite with clean version
+                open my $WF, q{>}, $fn
+                    or croak "Can not open $fn for writing!";
+                print {$WF} $ndigest{$f}
+                    or croak "print to [$fn] failed $1";
+                close $WF or croak "Unable to close $!";
+            }
+
+        }
+
+        return not $dirty;
+    }
+
 }    # END INSIDE-OUT CLASS
 
 1;
@@ -1237,7 +1327,7 @@ CipUX - Common CipUX functions
 
 =head1 VERSION
 
-version 3.4.0.5
+version 3.4.0.9
 
 =head1 SYNOPSIS
 
@@ -1421,7 +1511,7 @@ with uniq values.
 
 =head2 cfg_ext
 
-Gets all suppored extensions from Config::Any and remove non supported
+Gets all supported extensions from Config::Any and remove non supported
 extensions.
 
 =head2 cfg
@@ -1490,6 +1580,42 @@ for this task.
 Creates a directory 'cipux' under /var/cache if not present. This is used to
 store serialized objects.
 
+=head2 iterate_config_space
+
+Takes a single config file $cfg (can be undef, but must be present) and a
+string representing the configuration base $cfgbase. It will then just use $cfg
+or it will find out regarding to $cfgbase what files should be considered.
+
+  my $loadcfg_ar = $self->iterate_config_space(
+      {
+          cfg     => $cfg,
+          cfgbase => $cfgbase,
+      }
+  );
+
+returns a list of all configuration files which should be evaluated.
+
+=head2 evaluate_config_space
+
+Takes a araay ref to a list of configuration files $loadcfg_ar, preferably
+cunstructed by iterate_config_space. It also taks a string $cache_dir to the
+temporary location of a cache_dir. This value can be empty but not undef and
+has to be provided. It also require the $cfgbase string, the "domain" of the
+configuration.
+
+It will calculate for every config file the MD5sum for later comparison.
+
+        my $clean = $self->evaluate_config_space(
+            {
+                loadcfg_ar => $loadcfg_ar,
+                cache_dir  => $cache_dir,
+                cfgbase    => $cfgbase,
+            }
+        );
+
+It will return 0 if changed files are found and 1 if no changed files are
+found.
+
 
 =head1 DIAGNOSTICS
 
